// ============================================================================
// Module Name: IPat_DcFifoWrCtrl
// Revision: 20170930a
// Author: Altmo toolbox
// Description:
//  + Write adress control of IPat_DcFifo
// ============================================================================

`timescale 1ps/100fs

module IPat_DcFifoWrCtrl (
    clk,
    rst_n,
    i_wrreq,
    i_rdcadr,
    o_wrcadr,
    o_wradr,
    o_wren,
    o_wrfull,
    o_wrusedw
);

// parameter ==================================================================
parameter P_IPAT_DCFIFOWRCTRL_AWIDTH = 4;	// address width
parameter P_IPAT_DCFIFOWRCTRL_DBGMSG = "FALSE";	// debug message

// local parameter ============================================================
localparam LP_AWIDTH = P_IPAT_DCFIFOWRCTRL_AWIDTH;	// address width
localparam LP_DBGMSG = P_IPAT_DCFIFOWRCTRL_DBGMSG;	// debug message

localparam LP_UD = 1;	// unit delay to avoid racing

// synopsys translate_off
initial begin
    if (LP_DBGMSG == "TRUE") begin
        $display("INFO:%m:LP_AWIDTH=%0d",LP_AWIDTH);
    end
end
// synopsys translate_on


// port definition ============================================================
input			clk;		// clock
input			rst_n;		// reset 0 active
input			i_wrreq;	// write request
input	[LP_AWIDTH:0]	i_rdcadr;	// carry + read address

output	[LP_AWIDTH:0]	o_wrcadr;	// carry + write address
output	[LP_AWIDTH-1:0]	o_wradr;	// write address
output			o_wren;		// write enable to memory
output			o_wrfull;	// write full signal
output	[LP_AWIDTH:0]	o_wrusedw;	// used write word

// carry bit + write address --------------------------------------------------
reg	[LP_AWIDTH:0]	r_wrcadr;

always @(posedge clk or negedge rst_n) begin
    if (rst_n == 1'b0) begin
        r_wrcadr <= #LP_UD {(LP_AWIDTH+1'b1){1'b0}};
    end
    else begin
        r_wrcadr <= #LP_UD (o_wren)? (r_wrcadr + 1'b1) : r_wrcadr;
    end
end

assign o_wradr  = r_wrcadr[LP_AWIDTH-1:0]; // remove carry bit
assign o_wrcadr = r_wrcadr;

// write enbale ---------------------------------------------------------------
assign o_wren = ~o_wrfull & i_wrreq;

// write full -----------------------------------------------------------------
//assign o_wrfull
assign o_wrfull
  =   (r_wrcadr != i_rdcadr)                                // not same value (different carry)
    & (r_wrcadr[LP_AWIDTH-1:0] == i_rdcadr[LP_AWIDTH-1:0]); // same address

// write used word ------------------------------------------------------------
assign o_wrusedw = r_wrcadr - i_rdcadr;

endmodule
