// ============================================================================
// Module Name: IPat_FifoOutRst
// Revision: 20170924a
// Author: Altmo toolbox
// Description:
//  + Clear contol of output data
// ============================================================================

`timescale 1ps/100fs

module IPat_FifoOutRst (
    clk,
    i_q,
    o_q,
    i_rden,
    rst_n
);

// parameter ==================================================================
parameter P_IPAT_FIFOOUTRST_DWIDTH = 8;		// width of q
parameter P_IPAT_FIFOOUTRST_OUTFF = "FALSE";	// output ff enable

// local parameter ============================================================
localparam LP_DWIDTH = P_IPAT_FIFOOUTRST_DWIDTH;
localparam LP_OUTFF = P_IPAT_FIFOOUTRST_OUTFF;

localparam LP_UD = 1;	// unit delay to avoid racing

// port definition ============================================================
input			clk;	// clock
input	[LP_DWIDTH-1:0]	i_q;	// data input
output	[LP_DWIDTH-1:0]	o_q;	// data output
input			i_rden;	// read enable input
input			rst_n;	// reset 0 active


// register and wire definition ===============================================
reg	r_rden;

// logic ======================================================================

// Clear logic is released when next read comes.
always @(posedge clk or negedge rst_n) begin
    if (rst_n == 1'b0) begin
        r_rden <= #LP_UD 1'b0;
    end
    else begin
        r_rden <= #LP_UD (i_rden | r_rden);
    end
end

generate
    if (LP_OUTFF=="TRUE") begin
        reg	[LP_DWIDTH-1:0]	r_q;
        always @(posedge clk or negedge rst_n) begin
            if (rst_n == 1'b0) begin
                r_q <= #LP_UD {LP_DWIDTH{1'b0}};
            end
            else begin
                r_q <= #LP_UD (r_rden)? i_q : r_q;
            end
        end
        assign o_q = r_q;
    end
    else begin
        assign o_q = (r_rden)? i_q : {LP_DWIDTH{1'b0}};
    end
endgenerate

endmodule
