
// 20170930a
// Sanity checking for IPat_DcFifo.v

`timescale 1ps/100fs

module tb;

// parameter
parameter WCYC = 100*1000; // write clock cycle
parameter RCYC = 102*1000; // read clock cycle

parameter P_SAHEAD = "FALSE";  // readmode TRUE is Show-ahead

parameter P_DWIDTH = 2;  // data width
parameter P_AWIDTH = 3;  // address width
parameter P_WRPIPE = 2;  // write sync pipe
parameter P_RDPIPE = 2;  // read sync pipe
parameter P_OUTFF = "FALSE";
parameter P_DBGMSG = "FALSE";

parameter P_UD = 1; // unit delay

// clock
reg wrclk;
reg rdclk;
always #(WCYC/2) wrclk = ~wrclk;
always #(RCYC/2) rdclk = ~rdclk;
initial begin
    #(0) wrclk=0; rdclk=0;
end

// vcd dump
initial begin
    $dumpfile("wave.vcd");
    $dumpvars(0,tb);
end

reg [P_DWIDTH-1:0] data;
wire               wrreq;
wire               rdreq;
reg                rst_n;
reg [31:0]         wrrnd;
reg [31:0]         rdrnd;
wire               wrfull;


always @(posedge wrclk or negedge rst_n) begin
    if (~rst_n) begin
        data  <= #P_UD 1;
        wrrnd <= #P_UD 0;
    end
    else begin
        data  <= #P_UD ((wrreq === 1'b1) & (wrfull === 1'b0))? data + 1'b1 : data;
        wrrnd <= #P_UD $random;
    end
end

assign wrreq = wrrnd[0];

always @(posedge rdclk or negedge rst_n) begin
    if (~rst_n)
        rdrnd <= #P_UD 0;
    else
        rdrnd <= #P_UD $random;
end

assign rdreq = rdrnd[0];

reg	S_SAHEAD;

initial begin
    // read mode normal
    #(  0          ) rst_n=0;
    #(  1*1000*1000) rst_n=1;
    #(100*1000*1000) $finish;
end


// dut
IPat_DcFifo #(
    .P_IPAT_DCFIFO_DWIDTH(P_DWIDTH),
    .P_IPAT_DCFIFO_AWIDTH(P_AWIDTH),
    .P_IPAT_DCFIFO_WRPIPE(P_WRPIPE),
    .P_IPAT_DCFIFO_RDPIPE(P_RDPIPE),
    .P_IPAT_DCFIFO_SAHEAD(P_SAHEAD),
    .P_IPAT_DCFIFO_OUTFF(P_OUTFF),
    .P_IPAT_DCFIFO_DBGMSG(P_DBGMSG)
) dut (
    .wrclk(wrclk),
    .data(data),
    .wrreq(wrreq),
    .wrfull(wrfull),

    .rdclk(rdclk),
    .rdreq(rdreq),

    .rst_n(rst_n)
);

endmodule
